module Doable
  module Exceptions
    module Framework
      # Exception to be thrown when nothing else applies. Inherited by other exceptions.
      class GenericFrameworkError < StandardError
      end

      # When the provided parameters or other input is not valid.
      class InvalidInput < GenericFrameworkError
      end

      # Correct number of parameters is passed but it is not of the correct type.
      # i.e., type error in strongly typed languages
      class NotApplicable < InvalidInput
      end

      # This can be called when a helper method is expecting a parameter that wasn't passed.
      # i.e., an options hash doesn't contain the requisite elements. 
      class MissingParameter < InvalidInput
      end

      # This exception should be used by those rare features that aren't cross-platform
      class UnsupportedPlatformFeature < GenericFrameworkError
      end

      # !These exceptions should never be caught by anything!
      # This is an internal exception used primarily for logging purposes. Provides a clean way to log 
      class SkipStep < StandardError
      end

      # Allows the the application to notify the completion of a Rollback and that the job object was not successfully executed. 
      class RolledBack < StandardError
      end
    end
  end
end
